@@ -36,19 +36,10 @@ class REIXS_HDF5(object):
 
     def __init__(self, baseName, header_file):
         try:
-            file = os.path.join(baseName, header_file)
+            self.file = os.path.join(baseName, header_file)
         except:
             raise TypeError("You did not specify a directory path.")
 
-        # Load the HDF5 container
-        self.f = h5py.File(file, 'r')
-
-        # Create dictionary for scan numbers
-        self.scanIndexDictHeader = dict()
-        for k in self.f.keys():
-            if k.startswith("SCAN_"):
-                self.scanIndexDictHeader[int(k.split("_")[1])] = k
-
     def Scan(self, scan):
         return self.myScan(self, scan)
 
@@ -56,71 +47,79 @@ class REIXS_HDF5(object):
 
         def __init__(my, self, scan):
             try:
-                my.scan = self.scanIndexDictHeader[scan]
+                with h5py.File(self.file, 'r') as f:
+
+                    # Create dictionary for scan numbers
+                    scanIndexDictHeader = dict()
+                    for k in f.keys():
+                        if k.startswith("SCAN_"):
+                            scanIndexDictHeader[int(k.split("_")[1])] = k
+                    my.scan = scanIndexDictHeader[scan]
+
+                    try:
+                        my.mono_energy = np.array(f[f'{my.scan}/Data/beam'])
+                    except:
+                        raise TypeError("Problem detecting energy.")
+
+                    try:
+                        my.mesh_current = np.array(f[f'{my.scan}/Data/i0'])
+                    except:
+                        raise TypeError("Problem detecting mesh current.")
+
+                    try:
+                        my.sample_current = np.array(f[f'{my.scan}/Data/tey'])
+                    except:
+                        raise TypeError("Problem detecting sample current.")
+
+                    try:
+                        my.TEY = my.sample_current/my.mesh_current
+                    except:
+                        raise ValueError("Problem calculating TEY.")
+
+                    try:
+                        my.sdd_data = np.array(f[f'{my.scan}/Data/sddMCA'])
+                        my.sdd_energy = np.array(
+                            f[f'{my.scan}/Data/sddMCA_scale'])
+                    except:
+                        warnings.warn("Could not load SDD data / SDD energy scale")
+
+                    try:
+                        my.xeol_data = np.array(f[f'{my.scan}/Endstation/Detectors/XEOL/xeolMCA'])
+                        my.xeol_energy = np.array(f[f'{my.scan}/Endstation/Detectors/XEOL/xeolMCA_scale'])
+                        my.xeol_background = np.array(f[f'{my.scan}/Endstation/Detectors/XEOL/xeolMCA_back'])
+
+                    except:
+                        warnings.warn("Could not load XEOL data / XEOL emission scale")
+
+                    try:
+                        my.mcp_data = np.transpose(
+                            np.array(f[f'{my.scan}/Data/mcpMCA']))
+                        my.mcp_energy = np.array(
+                            f[f'{my.scan}/Data/mcpMCA_scale'])
+                    except:
+                        warnings.warn("Could not load MCP data / MCP energy scale")
+
+                    my.sca_data = pd.DataFrame()
+                    try:
+                        for entry in f[f'{my.scan}/Endstation/Counters']:
+                            my.sca_data[str(entry)] = np.array(
+                                f[f'{my.scan}/Endstation/Counters/{entry}'])
+                    except:
+                        raise UserWarning("Could not load SCAs from HDF5 container.")
+
             except:
                 raise ValueError("Scan not defined")
 
-            try:
-                my.mono_energy = np.array(self.f[f'{my.scan}/Data/beam'])
-            except:
-                raise TypeError("Problem detecting energy.")
-
-            try:
-                my.mesh_current = np.array(self.f[f'{my.scan}/Data/i0'])
-            except:
-                raise TypeError("Problem detecting mesh current.")
-
-            try:
-                my.sample_current = np.array(self.f[f'{my.scan}/Data/tey'])
-            except:
-                raise TypeError("Problem detecting sample current.")
-
-            try:
-                my.TEY = my.sample_current/my.mesh_current
-            except:
-                raise ValueError("Problem calculating TEY.")
-
-            try:
-                my.sdd_data = np.array(self.f[f'{my.scan}/Data/sddMCA'])
-                my.sdd_energy = np.array(
-                    self.f[f'{my.scan}/Data/sddMCA_scale'])
-            except:
-                warnings.warn("Could not load SDD data / SDD energy scale")
-
-            try:
-                my.mcp_data = np.transpose(
-                    np.array(self.f[f'{my.scan}/Data/mcpMCA']))
-                my.mcp_energy = np.array(
-                    self.f[f'{my.scan}/Data/mcpMCA_scale'])
-            except:
-                warnings.warn("Could not load MCP data / MCP energy scale")
-
-            my.sca_data = pd.DataFrame()
-            try:
-                for entry in self.f[f'{my.scan}/Endstation/Counters']:
-                    my.sca_data[str(entry)] = np.array(
-                        self.f[f'{my.scan}/Endstation/Counters/{entry}'])
-            except:
-                raise UserWarning("Could not load SCAs from HDF5 container.")
-
         def MCP_norm(my):
             """Normalize the counts of the MCP by incident flux at every given datapoint.
                This is only applied to EEMs."""
 
-            mcp_data_norm_init = []
-            for count, i in enumerate(my.mesh_current):
-                mcp_data_norm_i = np.true_divide(my.mcp_data[:, count], i)
-                mcp_data_norm_init.append(mcp_data_norm_i)
-            my.mcp_data_norm = np.array(mcp_data_norm_init)
+            my.mcp_data_norm = np.transpose(rixs_readutil.detector_norm(my.mcp_data,my.mesh_current))
 
             return my.mcp_data_norm
 
         def SDD_norm(my):
-            sdd_data_norm_init = list()
-            for i, mesh_current in enumerate(my.mesh_current):
-                sdd_data_norm_init.append(np.true_divide(
-                    my.sdd_data[i, :], mesh_current))
-            my.sdd_data_norm = np.array(sdd_data_norm_init)
+            my.sdd_data_norm = rixs_readutil.detector_norm(my.sdd_data,my.mesh_current[:,None])
 
             return my.sdd_data_norm
 
@@ -578,11 +577,7 @@ class REIXS_ASCII(object):
             """Normalize the counts of the MCP by incident flux at every given datapoint.
                This is only applied to EEMs."""
 
-            mcp_data_norm_init = []
-            for count, i in enumerate(my.mesh_current):
-                mcp_data_norm_i = np.true_divide(my.mcp_data[:, count], i)
-                mcp_data_norm_init.append(mcp_data_norm_i)
-            my.mcp_data_norm = (np.array(mcp_data_norm_init))
+            my.mcp_data_norm = np.transpose(rixs_readutil.detector_norm(my.mcp_data,my.mesh_current))
 
             return my.mcp_data_norm
 
@@ -600,11 +595,7 @@ class REIXS_ASCII(object):
             return rixs_readutil.rXES(my.mono_energy, my.mcp_data_norm, xes_incident_start, xes_incident_end)
 
         def SDD_norm(my):
-            sdd_data_norm_init = list()
-            for i, mesh_current in enumerate(my.mesh_current):
-                sdd_data_norm_init.append(np.true_divide(
-                    my.sdd_data[i, :], mesh_current))
-            my.sdd_data_norm = np.array(sdd_data_norm_init)
+            my.sdd_data_norm = rixs_readutil.detector_norm(my.sdd_data,my.mesh_current[:,None])
 
             return my.sdd_data_norm
 
