@@ -3,7 +3,7 @@ from itertools import groupby
 import pandas as pd
 import numpy as np
 from scipy.interpolate import interp1d
-
+from collections.abc import MutableMapping
 
 #########################################################################################
 def all_list_entries_equal(iterable):
@@ -22,12 +22,40 @@ def doesMatchPattern(string, patterns=[]):
 
     return False
 
+#########################################################################################
+def get_roi(roi):
+    try:
+        roi_low = float(roi.split(":")[0])
+        roi_high = float(roi.split(":")[1])
+    except:
+        roi_low = float(roi)
+        roi_high = float(roi)
+
+    return roi_low,roi_high
+
+#########################################################################################
+def check_idx(idx_low,idx_high):
+    if idx_low == idx_high:
+        idx_high = idx_low+1
+    
+    return idx_low,idx_high
+
 #########################################################################################
 def check_key_in_dict(key,dic):
     for k,v in dic.items():
         if key == k:
             return True
 
+#########################################################################################
+def flatten(d):
+    items = []
+    for k, v in d.items():
+        if isinstance(v, MutableMapping):
+            items.extend(flatten(v).items())
+        else:
+            items.append((k, v))
+    return dict(items)
+
 #########################################################################################            
 # Palette for Bokeh Plots
 
